import matplotlib.pyplot as plt
import numpy as np

from util.db_util import get_specific_bookmaker_odds
from util.model_input_util import create_feature_single_input, create_spi_input_sub_net, create_exp_input_sub_net
from util.util import result_tostring, vector_to_result

colors = ["-r", "-b", "-k", "-m"]


def evaluate_games_weekly(model, test_league_seasons, data, features_dict, strategies, dataframe=True, one_bet_only=False):
    game_week = {
        'odds': [],
        'probabilities': [],
        'results': []
    }
    game_weeks = []
    strategy_bankrolls = {}
    strategy_bankroll_histories = {}
    strategy_bet_histories = {}
    prob_outcome_dist = {
        "2016": [],
        "2017": []
    }
    if not dataframe:
        for season in test_league_seasons['seasons']:
            strategy_bankroll_histories[season] = {}
            for match in list(filter(lambda match: match['season'] == season, data)):
                game_week['results'].append(vector_to_result(match['result']))
                if "fatigue_factors" in features_dict:
                    input = create_exp_input_sub_net(match, features_dict)
                else:
                    input = create_feature_single_input(match, features_dict)
                probabilities = model.predict(input)
                result = match['result']
                prob_outcome_dist[season].append(list(zip(probabilities[0], result)))
                game_week['probabilities'].append(probabilities)
                game_week['odds'].append(get_specific_bookmaker_odds(match, "William Hill"))

                if len(game_week['odds']) == 10:
                    for strategy in strategies:
                        strategy.evaluate_game_week(game_week, one_bet_only=one_bet_only)

                    game_week = {
                        'odds': [],
                        'probabilities': [],
                        'results': []
                    }
            for idx, strategy in enumerate(strategies):
                strategy.report()
                strategy_bet_histories[strategy.name + "-" + str(season)] = strategy.betting_history
                strategy_bankrolls[strategy.name + "-" + str(season)] = strategy.bankroll
                strategy_bankroll_histories[season][strategy.name] = (strategy.bankroll_history, strategy.betting_history)
                strategy.reset()

    else:
        for season in test_league_seasons['seasons']:
            strategy_bankroll_histories[season] = {}
            for idx, match in data[data.season == season].iterrows():
                if 'psychology_factors' in features_dict:
                    model_input = create_spi_input_sub_net(match, features_dict)
                elif 'form_factors' in features_dict:
                    model_input = create_exp_input_sub_net(match, features_dict)
                else:
                    model_input = create_feature_single_input(match, features_dict)
                    #game_week['probabilities'].append(model.predict(([create_feature_single_input(match, features_dict)])))

                if not np.isnan(model_input).any():
                    game_week['results'].append(result_tostring(match.h_goals, match.a_goals))
                    probabilities = model.predict(model_input)
                    game_week['probabilities'].append(probabilities)
                    game_week['odds'].append(get_specific_bookmaker_odds(match, "William Hill", dataframe=True))
                    result = match['result']

                    prob_outcome_dist[season].append(list(zip(probabilities[0], result)))

                if (idx + 1) % 10 == 0 or idx == len(
                        data[features_dict['feature_list'] + ['a_goals', 'h_goals', 'id']].dropna() - 1):
                    for strategy in strategies:
                        strategy.evaluate_game_week(game_week, one_bet_only=one_bet_only)
                    game_week = {
                        'odds': [],
                        'probabilities': [],
                        'results': []
                    }
#            set_up_pyplot(season)
            for idx, strategy in enumerate(strategies):
                strategy.report()
                strategy_bet_histories[strategy.name + "-" + str(season)] = strategy.betting_history
                strategy_bankrolls[strategy.name + "-" + str(season)] = strategy.bankroll
                strategy_bankroll_histories[season][strategy.name] = (strategy.bankroll_history, strategy.betting_history)
                strategy.reset()

    return strategy_bankrolls, strategy_bankroll_histories, strategy_bet_histories, prob_outcome_dist


def evaluate_games_sequentially(model, data, features_dict, strategies):
    for idx, match in data[features_dict['feature_list'] + ['a_goals', 'h_goals', 'id']].dropna().iterrows():
        probabilities = model.predict([create_feature_single_input(match, features_dict)])
        odds = get_specific_bookmaker_odds(match, "William Hill")

        print("Probabilities: " + str(probabilities.tolist()[0]))
        print("Odds: " + str(odds))

        result = result_tostring(match.h_goals, match.a_goals)

        for strategy in strategies:
            strategy.evaluate_game(odds, probabilities.tolist()[0], result)
            append_bankroll_history_to_plot(strategy, color="-b")
        plt.show()


def append_bankroll_history_to_plot(strategy, color):
    fig = plt.figure(1)
    ax = fig.gca()
    ax.plot(range(strategy.games + 1), strategy.bankroll_history, color, label=strategy.name)
    plt.legend(bbox_to_anchor=(1.49, 0.65), loc="upper right")


def set_up_pyplot(season):
    fig = plt.figure(1)
    ax = fig.gca()
    ax.set_xlabel("Gameweek")
    ax.set_ylabel("Bankroll")
    ax.set_title("Season " + str(season))
    plt.tight_layout(rect=[0, 0, 0.75, 1])


def get_result_color(bet):
    if bet['result'] == bet['bet']:
        return 'g^'
    else:
        return 'ro'


def evaluate_baselines_by_gameweek(test_seasons, data, strategies, one_bet_only=True):
    strategy_bet_histories = {}
    strategy_bankroll_histories = {}
    for season in test_seasons:
        game_week = {
            'odds': [],
            'results': []
        }
        for match in list(filter(lambda match: match['season'] == season, data)):
            game_week['results'].append(vector_to_result(match['result']))
            odds = get_specific_bookmaker_odds(match, "William Hill", dataframe=False)
            game_week['odds'].append(odds)
            if len(game_week['odds']) == 10:
                for strategy in strategies:
                    strategy.evaluate_game_week(game_week, one_bet_only=one_bet_only)

                game_week = {
                    'odds': [],
                    'probabilities': [],
                    'results': []
                }
        for idx, strategy in enumerate(strategies):
            strategy.report()
            strategy_bet_histories[strategy.name + "-" + str(season)] = strategy.betting_history
            strategy_bankroll_histories[strategy.name + "-" + str(season)] = strategy.bankroll_history
            strategy.reset()
    return strategy_bankroll_histories, strategy_bet_histories